/***********************************************
 *  Seeduino XIAOを使ったセンサIOT
 * 　BME280の3種のデータをMQTTでパブリッシュ
 **********************************************/
#include <Wire.h>
#include <SparkFunBME280.h> 
#include <Adafruit_SSD1306.h>
// MQTT用メッセージ定義とインターバル変数
byte con[]={0x10,0x13,0x00,0x06,'M','Q','I','s','d','p',0x03,0x02,0x00,0x5c,0x00,0x05,'X','I','A','O','1'};
byte pub[]={0x30,0x1D,0x00,0x04,'T','e','s','t',0,0,0,0,0,0,0,',',0,0,0,0,0,',',0,0,0,0,0,',','E','N','D'};
int interval=0;
Adafruit_SSD1306 display(-1);
BME280 sensor;

/*** ソフトウェアリセット関数 ****/
void software_reset() {
  SCB->AIRCR = ((0x5FA << SCB_AIRCR_VECTKEY_Pos) | SCB_AIRCR_SYSRESETREQ_Msk);
} 

/*** ESP応答受信と指定時間待ちとシリアルモニタ出力 ***/
bool getResponse(String filter, int timeout){
    String data;
    char a, flag;
    unsigned long start = millis();

    flag = 0;
    while (millis() - start < timeout) {
        while(Serial1.available() > 0) {
          a = Serial1.read();
          Serial.write(a);
          if(a == '\0') continue;
            data += a;
        }
        if (data.indexOf(filter) != -1) {
            flag = 1;
            break;
        }   
    }
    if(flag == 1)
      return true;
    else
      return false;
}

/***** 設定関数 ***********/
void setup(){
  int i;
  
  Wire.begin();
  sensor.settings.I2CAddress = 0x76;    // BMEのI2Cアドレス指定
  sensor.beginI2C();                    // BMEのインスタンス生成
  pinMode(1, OUTPUT);                   // 目印LED
  pinMode(0, OUTPUT);
  Serial.begin(115200);                 // 通信速度指定
  Serial1.begin(115200); 
  while(!Serial1);
  delay(2000);
  // OLEDの電源電圧とI2Cアドレス設定
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C); 
  // MQTTブローカと接続確立 
  digitalWrite(1, HIGH);
  Serial.println("Call MQTT!");
  Serial1.print("AT+RST\r\n");
  delay(1000);  
  Serial1.print("AT+CWMODE=1\r\n"); // ステーションモード
  if(getResponse("OK", 1000)==false){Serial.println("NG\r\n");}   
  // APと接続
  do{
    delay(2000);  
    Serial1.print("AT+CWJAP=\"Buffalo-G-6370\",\"7tb7ksh8i44bc\"\r\n");
  }while(getResponse("WIFI GOT IP", 10000)==false);
  // MQTTブローカと接続
  do{
    Serial1.print("AT+CIPSTART=\"TCP\",\"192.168.11.51\",1883\r\n");
    delay(1000);
  }while(getResponse("busy", 2000));
  // MQTTコネクト要求送信
  Serial1.print("AT+CIPSEND=21\r\n");
  if(getResponse("OK", 1000)==false){Serial.println("NG\r\n");} 
  for (i=0;i<21;i++)
    Serial1.write(con[i]);                 
  if(getResponse("OK", 1000)==false){Serial.println("NG\r\n");} 
  digitalWrite(1, LOW);
}

/***** メインループ *******************/
void loop(){
  char s[7],t[5],u[5];
  int i;
  
  // OLED描画条件指定
  display.clearDisplay();           // 表示クリア
  display.setTextSize(1);           // 出力する文字の大きさ
  display.setTextColor(WHITE);      // 出力する文字の色  
  // センサデータOLED表示 
  display.setCursor(0, 0);          // 表示開始位置をホームにセット
  display.print("Press = ");
  display.print(sensor.readFloatPressure() / 100.0, 2);
  display.println(" hPa");
  display.setCursor(0, 11);         // 2行目
  display.print("Temp  = ");
  display.print(sensor.readTempC(), 2);
  display.println(" DegC");
  display.setCursor(0, 22);         // 3行目
  display.print("Humi  = ");
  display.print(sensor.readFloatHumidity(), 2);
  display.println(" %RH");
  display.display();                // 表示実行
  // 表示間隔とMQTT間隔の設定
  delay(2000);                      // 2秒間隔   
  interval++;                       // MQTT用間隔カウントアップ
  if(interval >= 15){
    interval = 0;                   // 30秒間隔
    digitalWrite(1, HIGH);
    // MQTTで送るデータに数値を埋め込む
    dtostrf((sensor.readFloatPressure()/100.0), 7, 2, s);
    dtostrf(sensor.readTempC(), 5, 2, t);
    dtostrf(sensor.readFloatHumidity(), 5, 2, u);
    memcpy(&pub[8], s, 7);               // 送信データ作成
    memcpy(&pub[16], t, 5);
    memcpy(&pub[22], u, 5);
    // MQTT送信実行 
    Serial.println("Send");              // 目印
    Serial1.print("AT+CIPSEND=31\r\n");  // 送信
    if(getResponse("OK", 1000)==false){software_reset();}  
    for (i=0;i<31;i++)
      Serial1.write(pub[i]);            // MQTTへパブリッシュ
    if(getResponse("OK", 1000)==false){software_reset();} 
    digitalWrite(1, LOW);
  }
}
